package javax.swing;

import java.awt.Point;
import java.security.AccessController;
import java.awt.Toolkit;
import java.security.PrivilegedAction;
import java.util.Iterator;
import java.awt.event.InputEvent;
import java.awt.AWTEvent;
import java.util.ArrayList;
import java.awt.event.AWTEventListener;
import javax.accessibility.AccessibleRole;
import javax.accessibility.AccessibleContext;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.awt.Dimension;
import java.beans.PropertyChangeEvent;
import java.awt.Graphics;
import javax.swing.border.Border;
import java.awt.LayoutManager;
import java.awt.Rectangle;
import sun.awt.AWTAccessor;
import javax.swing.plaf.LayerUI;
import javax.accessibility.Accessible;
import java.beans.PropertyChangeListener;
import java.awt.Component;

public final class JLayer<V extends Component> extends JComponent implements Scrollable, PropertyChangeListener, Accessible
{
    private V view;
    private LayerUI<? super V> layerUI;
    private JPanel glassPane;
    private long eventMask;
    private transient boolean isPainting;
    private transient boolean isPaintingImmediately;
    private static final JLayer.LayerEventController eventController;
    
    public JLayer() {
        this(null);
    }
    
    public JLayer(V v) {
        this(v, new LayerUI());
    }
    
    public JLayer(V view, LayerUI<V> ui) {
        this.setGlassPane(this.createGlassPane());
        this.setView(view);
        this.setUI(ui);
    }
    
    public V getView() {
        return this.view;
    }
    
    public void setView(V v) {
        Component view = this.getView();
        if (view != null) {
            super.remove(view);
        }
        if (v != null) {
            super.addImpl(v, null, this.getComponentCount());
        }
        this.firePropertyChange("view", view, this.view = v);
        this.revalidate();
        this.repaint();
    }
    
    public void setUI(LayerUI<? super V> layerUI) {
        super.setUI(this.layerUI = layerUI);
    }
    
    public LayerUI<? super V> getUI() {
        return this.layerUI;
    }
    
    public JPanel getGlassPane() {
        return this.glassPane;
    }
    
    public void setGlassPane(JPanel panel) {
        JPanel glassPane = this.getGlassPane();
        boolean visible = false;
        if (glassPane != null) {
            visible = glassPane.isVisible();
            super.remove(glassPane);
        }
        if (panel != null) {
            AWTAccessor.getComponentAccessor().setMixingCutoutShape(panel, new Rectangle());
            panel.setVisible((boolean)visible);
            super.addImpl(panel, null, 0);
        }
        this.firePropertyChange("glassPane", glassPane, this.glassPane = panel);
        this.revalidate();
        this.repaint();
    }
    
    public JPanel createGlassPane() {
        return new JLayer.DefaultLayerGlassPane();
    }
    
    @Override
    public void setLayout(LayoutManager layoutManager) {
        if (layoutManager != null) {
            throw new IllegalArgumentException("JLayer.setLayout() not supported");
        }
        return;
    }
    
    @Override
    public void setBorder(Border border) {
        if (border != null) {
            throw new IllegalArgumentException("JLayer.setBorder() not supported");
        }
        return;
    }
    
    @Override
    protected void addImpl(Component component, Object o, int n) {
        throw new UnsupportedOperationException("Adding components to JLayer is not supported, use setView() or setGlassPane() instead");
    }
    
    @Override
    public void remove(Component component) {
        if (component == null) {
            super.remove(component);
        }
        else if (component == this.getView()) {
            this.setView(null);
        }
        else if (component == this.getGlassPane()) {
            this.setGlassPane(null);
        }
        else {
            super.remove(component);
        }
    }
    
    @Override
    public void removeAll() {
        if (this.view != null) {
            this.setView(null);
        }
        if (this.glassPane != null) {
            this.setGlassPane(null);
        }
    }
    
    @Override
    protected boolean isPaintingOrigin() {
        return true;
    }
    
    @Override
    public void paintImmediately(int x, int y, int width, int height) {
        if (!this.isPaintingImmediately && this.getUI() != null) {
            this.isPaintingImmediately = true;
            try {
                this.getUI().paintImmediately(x, y, width, height, this);
            }
            finally {
                this.isPaintingImmediately = false;
            }
        }
        else {
            super.paintImmediately(x, y, width, height);
        }
    }
    
    @Override
    public void paint(Graphics graphics) {
        if (!this.isPainting) {
            this.isPainting = true;
            try {
                super.paintComponent(graphics);
            }
            finally {
                this.isPainting = false;
            }
        }
        else {
            super.paint(graphics);
        }
    }
    
    @Override
    protected void paintComponent(Graphics graphics) {
    }
    
    @Override
    public boolean isOptimizedDrawingEnabled() {
        return false;
    }
    
    @Override
    public void propertyChange(PropertyChangeEvent evt) {
        if (this.getUI() != null) {
            this.getUI().applyPropertyChange(evt, this);
        }
    }
    
    public void setLayerEventMask(long eventMask) {
        long layerEventMask = this.getLayerEventMask();
        this.firePropertyChange("layerEventMask", layerEventMask, this.eventMask = eventMask);
        if (eventMask != layerEventMask) {
            this.disableEvents(layerEventMask);
            this.enableEvents(this.eventMask);
            if (this.isDisplayable()) {
                JLayer.LayerEventController.access$100(JLayer.eventController, layerEventMask, eventMask);
            }
        }
    }
    
    public long getLayerEventMask() {
        return this.eventMask;
    }
    
    @Override
    public void updateUI() {
        if (this.getUI() != null) {
            this.getUI().updateUI(this);
        }
    }
    
    @Override
    public Dimension getPreferredScrollableViewportSize() {
        if ((this.getView() instanceof Scrollable)) {
            return ((Scrollable)this.getView()).getPreferredScrollableViewportSize();
        }
        return this.getPreferredSize();
    }
    
    @Override
    public int getScrollableBlockIncrement(Rectangle rectangle, int n, int n2) {
        if ((this.getView() instanceof Scrollable)) {
            return ((Scrollable)this.getView()).getScrollableBlockIncrement(rectangle, n, n2);
        }
        return (n == 1) ? rectangle.height : rectangle.width;
    }
    
    @Override
    public boolean getScrollableTracksViewportHeight() {
        return this.getView() instanceof Scrollable && ((Scrollable)this.getView()).getScrollableTracksViewportHeight();
    }
    
    @Override
    public boolean getScrollableTracksViewportWidth() {
        return this.getView() instanceof Scrollable && ((Scrollable)this.getView()).getScrollableTracksViewportWidth();
    }
    
    @Override
    public int getScrollableUnitIncrement(Rectangle rectangle, int n, int n2) {
        if ((this.getView() instanceof Scrollable)) {
            return ((Scrollable)this.getView()).getScrollableUnitIncrement(rectangle, n, n2);
        }
        return 1;
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        if (this.layerUI != null) {
            this.setUI(this.layerUI);
        }
        if (this.eventMask != 0L) {
            JLayer.LayerEventController.access$100(JLayer.eventController, 0L, this.eventMask);
        }
    }
    
    @Override
    public void addNotify() {
        super.addNotify();
        JLayer.LayerEventController.access$100(JLayer.eventController, 0L, this.eventMask);
    }
    
    @Override
    public void removeNotify() {
        super.removeNotify();
        JLayer.LayerEventController.access$100(JLayer.eventController, this.eventMask, 0L);
    }
    
    @Override
    public void doLayout() {
        if (this.getUI() != null) {
            this.getUI().doLayout(this);
        }
    }
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.accessibleContext == null) {
            this.accessibleContext = new JComponent.AccessibleJComponent() {
                @Override
                public AccessibleRole getAccessibleRole() {
                    return AccessibleRole.PANEL;
                }
            };
        }
        return this.accessibleContext;
    }
    
    static {
        eventController = new JLayer.LayerEventController();
    }
    
    private static class LayerEventController implements AWTEventListener
    {
        private ArrayList<Long> layerMaskList;
        private long currentEventMask;
        private static final long ACCEPTED_EVENTS = 231487L;
        
        private LayerEventController() {
            this.layerMaskList = new ArrayList();
        }
        
        @Override
        public void eventDispatched(AWTEvent e) {
            Object source = e.getSource();
            if ((source instanceof Component)) {
                for (Component parent = (Component)source; parent != null; parent = parent.getParent()) {
                    if ((parent instanceof JLayer)) {
                        JLayer<Object> l = (JLayer)parent;
                        LayerUI<Object> ui = l.getUI();
                        if (ui != null && this.isEventEnabled(l.getLayerEventMask(), e.getID()) && (!(e instanceof InputEvent) || !((InputEvent)e).isConsumed())) {
                            ui.eventDispatched(e, l);
                        }
                    }
                }
            }
        }
        
        private void updateAWTEventListener(long l, long i) {
            if (l != 0L) {
                this.layerMaskList.remove(Long.valueOf(l));
            }
            if (i != 0L) {
                this.layerMaskList.add(Long.valueOf(i));
            }
            long n = 0L;
            Iterator<Long> iterator = this.layerMaskList.iterator();
            while (iterator.hasNext()) {
                n |= (long)iterator.next();
            }
            long currentEventMask = n & 0x3883FL;
            if (currentEventMask == 0L) {
                this.removeAWTEventListener();
            }
            else if (this.getCurrentEventMask() != currentEventMask) {
                this.removeAWTEventListener();
                this.addAWTEventListener(currentEventMask);
            }
            this.currentEventMask = currentEventMask;
        }
        
        private long getCurrentEventMask() {
            return this.currentEventMask;
        }
        
        private void addAWTEventListener(long n) {
            AccessController.doPrivileged(new PrivilegedAction<Void>() {
                @Override
                public Void run() {
                    Toolkit.getDefaultToolkit().addAWTEventListener(JLayer.LayerEventController.this, n);
                    return null;
                }
            });
        }
        
        private void removeAWTEventListener() {
            AccessController.doPrivileged(new PrivilegedAction<Void>() {
                @Override
                public Void run() {
                    Toolkit.getDefaultToolkit().removeAWTEventListener(JLayer.LayerEventController.this);
                    return null;
                }
            });
        }
        
        private boolean isEventEnabled(long n, int n2) {
            return ((n & 0x1L) != 0x0L && n2 >= 100 && n2 <= 103) || ((n & 0x2L) != 0x0L && n2 >= 300 && n2 <= 301) || ((n & 0x4L) != 0x0L && n2 >= 1004 && n2 <= 1005) || ((n & 0x8L) != 0x0L && n2 >= 400 && n2 <= 402) || ((n & 0x20000L) != 0x0L && n2 == 507) || ((n & 0x20L) != 0x0L && (n2 == 503 || n2 == 506)) || ((n & 0x10L) != 0x0L && n2 != 503 && n2 != 506 && n2 != 507 && n2 >= 500 && n2 <= 507) || ((n & 0x800L) != 0x0L && n2 >= 1100 && n2 <= 1101) || ((n & 0x8000L) != 0x0L && n2 == 1400) || ((n & 0x10000L) != 0x0L && (n2 == 1401 || n2 == 1402));
        }
        
        static void access$100(JLayer.LayerEventController layerEventController, long n, long n2) {
            layerEventController.updateAWTEventListener(n, n2);
        }
    }
    
    private static class DefaultLayerGlassPane extends JPanel
    {
        public DefaultLayerGlassPane() {
            this.setOpaque(false);
        }
        
        @Override
        public boolean contains(int x, int y) {
            int i = 0;
            while (i < this.getComponentCount()) {
                Component component = this.getComponent(i);
                Point convertPoint = SwingUtilities.convertPoint(this, new Point(x, y), component);
                if (component.isVisible() && component.contains(convertPoint)) {
                    return true;
                }
                ++i;
            }
            return (this.getMouseListeners().length != 0 || this.getMouseMotionListeners().length != 0 || this.getMouseWheelListeners().length != 0 || this.isCursorSet()) && super.contains(x, y);
        }
    }
}
